@@ -1,9 +1,8 @@
 /*
- * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
@@ -22,6 +21,7 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
 #include <libkern/OSByteOrder.h>
 
 extern "C" {
@@ -89,9 +89,13 @@ AppleUSBOHCI::CreateGeneralTransfer(
 
     // 5-14-02 JRH
     // 2905718
-    // before we do anything, check to make sure that the endpoint is not halted. if it is, return an immediate error
-    if (USBToHostLong(queue->pShared->tdQueueHeadPtr) & kOHCIHeadPointer_H)
+    // before we do anything, check to make sure that the endpoint is not halted. if it is, return an immediate error.
+    // 4-2-03 FAU
+    // Don't do this for a control transaction to endpoint 0
+    //
+    if ( (USBToHostLong(queue->pShared->tdQueueHeadPtr) & kOHCIHeadPointer_H) && !( (type == kOHCIControlSetupType) && ( ((USBToHostLong(queue->pShared->flags) & kOHCIEDControl_EN) >> kOHCIEDControl_ENPhase) == 0 )) )
     {
+        
         USBError(1, "%s[%p]::CreateGeneralTransfer - trying to queue to a stalled pipe", getName(), this);
         status = kIOUSBPipeStalled;
     }
@@ -922,10 +926,11 @@ AppleUSBOHCI::UIMCreateIsochTransfer(
             // Handy for debugging transfer lists
             itdFlags |= (kOHCIGTDConditionNotAccessed << kOHCIGTDControl_CCPhase);
             
-            // Set the DI bits (Delay Interrupt) to 111b on all but the last TD
-            // (this means that only the last TD will generate an interrupt)
-            //
-            itdFlags |= ( 0x7 << kOHCIGTDControl_DIPhase );
+            // In the past, we set the DI bits to 111b at this point, so that we only interrupted in the last transfer.  
+            // However, this presents a problem when we abort a pipe, there might be a partial transfer 
+            // in the done queue that has not caused an interrupt.  Then  when another TD completes, we will
+            // process this orphaned  TD and update the framelist the memory could have been released in the meantime, since
+            // our client could have closed the connection.  So, we need to interrupt for every TD.
             
             OSWriteLittleInt32(&pTailITD->pShared->flags, 0, itdFlags);
 
@@ -1033,10 +1038,19 @@ AppleUSBOHCI::UIMAbortEndpoint(
 
     pED->pShared->flags |= HostToUSBLong(kOHCIEDControl_K);	// mark the ED as skipped
 
-    // We used to wait for a SOF interrupt here.  Now just sleep for 1 ms.
+    // We used to wait for a SOF interrupt here.  Now just sleep for 2 ms:  1 to finish processing
+    // the frame and 1 to let the filter interrupt routine finish
     //
-    IOSleep(1);
-    
+    IOSleep(2);
+
+    // Process the Done Queue in case there is something there
+    //
+    if (_writeDoneHeadInterrupt & kOHCIHcInterrupt_WDH)
+    {
+        _writeDoneHeadInterrupt = 0;
+        UIMProcessDoneQueue(NULL);
+    }
+
     RemoveTDs(pED);
 
     pED->pShared->flags &= ~HostToUSBLong(kOHCIEDControl_K);	// activate ED again
@@ -1109,10 +1123,19 @@ AppleUSBOHCI::UIMDeleteEndpoint(
 
     _pOHCIRegisters->hcControl = HostToUSBLong(hcControl);
 
-    // We used to wait for a SOF interrupt here.  Now just sleep for 1 ms.
+    // We used to wait for a SOF interrupt here.  Now just sleep for 2 ms:  1 to finish processing
+    // the frame and 1 to let the filter interrupt routine finish
     //
-    IOSleep(1);
+    IOSleep(2);
 
+    // Process the Done Queue in case there is something there
+    //
+    if (_writeDoneHeadInterrupt & kOHCIHcInterrupt_WDH)
+    {
+        _writeDoneHeadInterrupt = 0;
+        UIMProcessDoneQueue(NULL);
+    }
+    
     // restart hcControl
     hcControl |= controlMask;
     _pOHCIRegisters->hcControl = HostToUSBLong(hcControl);
@@ -2224,20 +2247,17 @@ AppleUSBOHCI::UIMCreateIsochTransfer(
             // Handy for debugging transfer lists
             itdFlags |= (kOHCIGTDConditionNotAccessed << kOHCIGTDControl_CCPhase);
             
-            // Set the DI bits (Delay Interrupt) to 111b on all but the last TD
-            // (this means that only the last TD will generate an interrupt)
-            //
-            if ( !(useUpdateFrequency && (curFrameInTD >= updateFrequency)) || !useUpdateFrequency )
-            {
-                USBLog(7, "%s[%p]::UIMCreateIsochTransfer(LL) - Seting DI bits to 111b (curFrameInRequest %d)", getName(), this, curFrameInRequest);
-                itdFlags |= ( 0x7 << kOHCIGTDControl_DIPhase );
-            }
+            // In the past, we set the DI bits to 111b at this point, so that we only interrupted in the last transfer.  However, this presents a problem
+            // for low latency in the case of a transfer having multiple TDs:  When we abort a pipe, there might be a partial transfer 
+            // in the done queue that has not caused an interrupt.  If we then unmap the frame list (because our user client died), then
+            // when another TD completes, we will process this orphaned low latency TD and hang/panic 'cause we'll try to access the unmapped memory.
+            // Hence, we need to make sure that we interrupt after every TD.
            
             curFrameInTD = 0;
             needNewITD = true;	// To simplify test at top of loop.
 
             OSWriteLittleInt32(&pTailITD->pShared->flags, 0, itdFlags);
-            
+
             pTailITD->completion.action = NULL;
             pTailITD = pTailITD->pLogicalNext;		// this is the "old" pNewTD
             OSWriteLittleInt32(&pTailITD->pShared->nextTD, 0, pNewITD->pPhysical);	// link to the "new" pNewTD
